package edu.mit.ll.group43.surfaceoptimization.aircraft;

/**
 * Represents an aircraft, whose file and FCFS indexes, spot ready time,
 * unimpeded taxi time, and weight class are given. Its optimal spot
 * release time is solved for and set by a scheduling algorithm, which
 * takes the inputs of each aircraft into account.
 * 
 * @author William Hawkins
 *
 */
public class Aircraft implements Cloneable, Comparable<Aircraft> {
	
	private int fileIndex;
	private int fcfsIndex;
	private int spotReadyTime;
	private int unimpTaxiTime;
	
	private static int created = 0;
	private WeightClass weightClass;
	
	private int optReleaseTime;
	private int optTakeOffIndex;
	
	public Aircraft(WeightClass weightClass, int fileIndex, int spotReadyTime,
			int unimpTaxiTime) {
		this.weightClass = weightClass;
		this.fileIndex = fileIndex;
		this.spotReadyTime = spotReadyTime;
		this.unimpTaxiTime = unimpTaxiTime;
		
		this.optReleaseTime = 0;
		this.setOptTakeOffIndex(0);
		created ++;
	}
	
	public static int getCreated() {
		return created;
	}

	public int getOptReleaseTime() {
		return optReleaseTime;
	}

	public void setOptReleaseTime(int optReadyTime) {
		this.optReleaseTime = optReadyTime;
	}

	public WeightClass getWeightClass() {
		return weightClass;
	}

	public int getSpotReadyTime() {
		return spotReadyTime;
	}

	public int getUnimpTaxiTime() {
		return unimpTaxiTime;
	}
	
	public int getFileIndex() {
		return fileIndex;
	}

	public int getFcfsIndex() {
		return fcfsIndex;
	}
	
	public void setFcfsIndex(int fcfsIndex) {
		this.fcfsIndex = fcfsIndex;
	}
	
	public int getRunwayReadyTime() {
		return this.getOptReleaseTime() + this.getUnimpTaxiTime();
	}
	
	@Override
	public int compareTo(Aircraft other) {
		if (this.equals(other)) {
			return 0;
		}
		return this.fcfsIndex - other.getFcfsIndex();
	}
	
	@Override
	public boolean equals(Object other) {
		if (this == other) {
			return true;
		} else if (other == null) {
			return false;
		}
		return this.fcfsIndex == ((Aircraft) other).getFcfsIndex();
	}
	
	@Override
	public int hashCode() {
		int hash = 1;
		hash = hash * 31 + fcfsIndex;
		return hash;
	}
	
	@Override
	public Aircraft clone() {
		Aircraft ac = new Aircraft(this.weightClass, this.fileIndex, this.spotReadyTime, this.unimpTaxiTime);
		ac.setFcfsIndex(this.fcfsIndex);
		ac.setOptReleaseTime(this.optReleaseTime);
		return ac;
	}
	
	@Override
	public String toString() {
		return "" + this.fcfsIndex;
	}

	public int getOptTakeOffIndex() {
		return optTakeOffIndex;
	}

	public void setOptTakeOffIndex(int optTakeOffIndex) {
		this.optTakeOffIndex = optTakeOffIndex;
	}
}
