package edu.mit.ll.group43.surfaceoptimization.dp.factory;

import edu.mit.ll.group43.surfaceoptimization.aircraft.MinimumSeparationCalculator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.WeightClass;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.cost.CostStrategy;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.edge.EdgeValueStrategy;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.netgen.CPSNetworkGenerator;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.node.CPSNetworkNodeFactory;
import edu.mit.ll.group43.surfaceoptimization.dp.factory.sequencerecovery.SequenceRecoveryStrategy;

/**
 * Abstract class to be extended by classes that serve as factories
 * for the DPSequencer. These factories provide it with pieces of DP
 * algorithms that the DPSequencer uses to optimally re-sequence the
 * FCFS sequence that it is passed. Essentially, each factory
 * represents its own DP algorithm for solving the spot scheduling
 * problem.
 * 
 * @author William Hawkins
 *
 */
public abstract class DPSequencerFactory {

	protected int cps;
	protected MinimumSeparationCalculator<? extends WeightClass> calc;
	
	public DPSequencerFactory(int cps, MinimumSeparationCalculator<? extends WeightClass> calc) {
		this.cps = cps;
		this.calc = calc;
	}
	
	public int getCPS() {
		return cps;
	}
	
	public MinimumSeparationCalculator<? extends WeightClass> getCalc() {
		return calc;
	}
	
	public abstract EdgeValueStrategy getEdgeValueStrategy();
	
	public abstract CostStrategy getCostStrategy();
	
	public abstract CPSNetworkNodeFactory getCPSNetworkNodeFactory();
	
	public abstract SequenceRecoveryStrategy getSequenceRecoveryStrategy();
	
	public abstract CPSNetworkGenerator getCPSNetworkGenerator();
}
