package edu.mit.ll.group43.surfaceoptimization.dp.factory.cost;

import java.util.ArrayList;
import java.util.TreeSet;

import edu.mit.ll.group43.surfaceoptimization.aircraft.MinimumSeparationCalculator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.WeightClass;
import edu.mit.ll.group43.surfaceoptimization.network.NetworkEdge;
import edu.mit.ll.group43.surfaceoptimization.network.dp.CPSNetworkNode;

/**
 * Cost strategy for a dynamic programming sequencer that maximizes
 * throughput at the runway.
 * 
 * NOTE: Not fully implemented, and does not solve the same problem
 * as other cost strategies. Could be used in the future to perform
 * a second optimization at the runway if runway delay is incurred
 * due to stochastic unimpeded taxi times.
 * 
 * @author William Hawkins
 *
 */
public class RunwayCostStrategy extends CostStrategy {

	public RunwayCostStrategy(int cps,
			MinimumSeparationCalculator<? extends WeightClass> calc) {
		super(cps, calc);
	}

	/**
	 * Initializes all nodes in the first stage of the network to
	 * have zero distance from the beginning and to have a null
	 * previous node.
	 * 
	 * @param network
	 */
	@Override
	public void initializeFirstStage(ArrayList<TreeSet<CPSNetworkNode>> network) {
		for (CPSNetworkNode node : network.get(0)) {
			node.setOptDistance(0);
			node.setOptPrevNode(null);
		}
	}

	/**
	 * Sets the distance from the beginning and the optimal
	 * previous edge that allows this distance to be achieved.
	 * 
	 * @param edge
	 * @param distance
	 */
	@Override
	public void setOptimalValues(NetworkEdge edge, int distance) {
		CPSNetworkNode node = (CPSNetworkNode) edge.getTarget();
		node.setOptDistance(distance);
		node.setOptPrevNode(edge);
	}
	
	/**
	 * Returns the minimum separation time between the source and
	 * target nodes of the given edge.
	 * 
	 * @param edge
	 */
	@Override
	public int getEdgeWeight(NetworkEdge edge) {
		return edge.getWeight();
	}
}
