package edu.mit.ll.group43.surfaceoptimization.dp.factory.cost;

import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Collections;

import junit.framework.TestCase;

import org.junit.Before;
import org.junit.Test;

import edu.mit.ll.group43.surfaceoptimization.aircraft.Aircraft;
import edu.mit.ll.group43.surfaceoptimization.aircraft.AircraftAlphaComparator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.AircraftSpotReleaseComparator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.DefaultDepartureMinimumSeparationCalculator;
import edu.mit.ll.group43.surfaceoptimization.aircraft.DefaultWeightClass;
import edu.mit.ll.group43.surfaceoptimization.measurement.Measurer;
import edu.mit.ll.group43.surfaceoptimization.measurement.RunwaySystemDelayMeasurer;

public class AllGapsForceSpotCostStrategyTest extends TestCase {

	private AllGapsForceSpotCostStrategy strategy;
	
	@Override
	@Before
	public void setUp() {
		try {
			strategy = new AllGapsForceSpotCostStrategy(1, DefaultDepartureMinimumSeparationCalculator.getInstance());
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
	}
	
	private ArrayList<Aircraft> createTakeOffSequence(int[] indexes, int[] spotReadyTimes,
			int[] optReleaseTimes) {
		ArrayList<Aircraft> takeOffSequence = new ArrayList<Aircraft>();
		for (int i = 0; i < indexes.length; i ++) {
			Aircraft ac = new Aircraft(DefaultWeightClass.LARGE, indexes[i], spotReadyTimes[i], 100);
			ac.setFcfsIndex(indexes[i]);
			ac.setOptReleaseTime(optReleaseTimes[i]);
			takeOffSequence.add(ac);
		}
		Collections.sort(takeOffSequence, AircraftAlphaComparator.getInstance());
		return takeOffSequence;
	}
	
	private void testAdjustment(ArrayList<Aircraft> takeOffSequence, int expDiffDelay,
			int[] expIndDelays) throws FileNotFoundException {
		
		assertEquals(strategy.adjustTakeOffs(takeOffSequence, 1, takeOffSequence.size()), expDiffDelay);
		for (int i = 0; i < takeOffSequence.size(); i ++) {
			assertEquals(takeOffSequence.get(i).getSpotReadyTime() + expIndDelays[i],
					takeOffSequence.get(i).getOptReleaseTime());
		}	
		
		Collections.sort(takeOffSequence, AircraftSpotReleaseComparator.getInstance());
		Measurer m = new RunwaySystemDelayMeasurer();
		assertEquals(0, m.measure(takeOffSequence));
	}
	
	@Test
	public void testAdjustTakeOffs() throws FileNotFoundException {
		int[] indexes = {1, 2, 3, 4, 5};
		int[] spotReadyTimes = {0, 10, 20, 30, 40};
		int[] optReleaseTimes = {0, 10, 20, 30, 40};
		
		// 0 + 100 = 100
		// 10 + 100 = 110
		// 20 + 100 = 120
		// 30 + 100 = 130
		// 40 + 10 = 140
		
		// 1 leaves on time at 0 (0, 160)
		// 2 must now leave at 60 (50, 220)
		// 3 must now leave at 120 (100, 280)
		// 4 must now leave at 180 (150, 340)
		// 5 must now leave at 240 (200, 400)
		// Total diff delay = 0 + 50 + 100 + 150 + 200 = 500
		int[] expIndDelays = {0, 50, 100, 150, 200};
		ArrayList<Aircraft> takeOffSequence = createTakeOffSequence(indexes, spotReadyTimes, optReleaseTimes);
		testAdjustment(takeOffSequence, 500, expIndDelays);
	}
}
